@@ -10,14 +10,20 @@ Independent Component Analysis, by  Hyvarinen et al.
 # License: BSD 3 clause
 
 import warnings
+from numbers import Integral, Real
 
 import numpy as np
 from scipy import linalg
 
-from ..base import BaseEstimator, TransformerMixin, _ClassNamePrefixFeaturesOutMixin
+from ..base import (
+    BaseEstimator,
+    ClassNamePrefixFeaturesOutMixin,
+    TransformerMixin,
+    _fit_context,
+)
 from ..exceptions import ConvergenceWarning
-
-from ..utils import check_array, as_float_array, check_random_state
+from ..utils import as_float_array, check_array, check_random_state
+from ..utils._param_validation import Interval, Options, StrOptions, validate_params
 from ..utils.validation import check_is_fitted
 
 __all__ = ["fastica", "FastICA"]
@@ -112,14 +118,18 @@ def _ica_par(X, tol, g, fun_args, max_iter, w_init):
         W1 = _sym_decorrelation(np.dot(gwtx, X.T) / p_ - g_wtx[:, np.newaxis] * W)
         del gwtx, g_wtx
         # builtin max, abs are faster than numpy counter parts.
-        lim = max(abs(abs(np.diag(np.dot(W1, W.T))) - 1))
+        # np.einsum allows having the lowest memory footprint.
+        # It is faster than np.diag(np.dot(W1, W.T)).
+        lim = max(abs(abs(np.einsum("ij,ij->i", W1, W)) - 1))
         W = W1
         if lim < tol:
             break
     else:
         warnings.warn(
-            "FastICA did not converge. Consider increasing "
-            "tolerance or the maximum number of iterations.",
+            (
+                "FastICA did not converge. Consider increasing "
+                "tolerance or the maximum number of iterations."
+            ),
             ConvergenceWarning,
         )
 
@@ -151,17 +161,27 @@ def _cube(x, fun_args):
     return x**3, (3 * x**2).mean(axis=-1)
 
 
+@validate_params(
+    {
+        "X": ["array-like"],
+        "return_X_mean": ["boolean"],
+        "compute_sources": ["boolean"],
+        "return_n_iter": ["boolean"],
+    },
+    prefer_skip_nested_validation=False,
+)
 def fastica(
     X,
     n_components=None,
     *,
     algorithm="parallel",
-    whiten="warn",
+    whiten="unit-variance",
     fun="logcosh",
     fun_args=None,
     max_iter=200,
     tol=1e-04,
     w_init=None,
+    whiten_solver="svd",
     random_state=None,
     return_X_mean=False,
     compute_sources=True,
@@ -180,24 +200,23 @@ def fastica(
         `n_features` is the number of features.
 
     n_components : int, default=None
-        Number of components to extract. If None no dimension reduction
-        is performed.
+        Number of components to use. If None is passed, all are used.
 
     algorithm : {'parallel', 'deflation'}, default='parallel'
-        Apply a parallel or deflational FASTICA algorithm.
+        Specify which algorithm to use for FastICA.
 
-    whiten : str or bool, default="warn"
+    whiten : str or bool, default='unit-variance'
         Specify the whitening strategy to use.
-        If 'arbitrary-variance'  (default), a whitening with variance arbitrary is used.
-        If 'unit-variance', the whitening matrix is rescaled to ensure that each
-        recovered source has unit variance.
-        If False, the data is already considered to be whitened, and no
-        whitening is performed.
 
-        .. deprecated:: 1.1
-            From version 1.3, `whiten='unit-variance'` will be used by default.
-            `whiten=True` is deprecated from 1.1 and will raise ValueError in 1.3.
-            Use `whiten=arbitrary-variance` instead.
+        - If 'arbitrary-variance', a whitening with variance
+          arbitrary is used.
+        - If 'unit-variance', the whitening matrix is rescaled to ensure that
+          each recovered source has unit variance.
+        - If False, the data is already considered to be whitened, and no
+          whitening is performed.
+
+        .. versionchanged:: 1.3
+            The default value of `whiten` changed to 'unit-variance' in 1.3.
 
     fun : {'logcosh', 'exp', 'cube'} or callable, default='logcosh'
         The functional form of the G function used in the
@@ -206,10 +225,10 @@ def fastica(
         You can also provide your own function. It should return a tuple
         containing the value of the function, and of its derivative, in the
         point. The derivative should be averaged along its last dimension.
-        Example:
+        Example::
 
-        def my_g(x):
-            return x ** 3, np.mean(3 * x ** 2, axis=-1)
+            def my_g(x):
+                return x ** 3, (3 * x ** 2).mean(axis=-1)
 
     fun_args : dict, default=None
         Arguments to send to the functional form.
@@ -219,13 +238,25 @@ def fastica(
     max_iter : int, default=200
         Maximum number of iterations to perform.
 
-    tol : float, default=1e-04
+    tol : float, default=1e-4
         A positive scalar giving the tolerance at which the
         un-mixing matrix is considered to have converged.
 
     w_init : ndarray of shape (n_components, n_components), default=None
-        Initial un-mixing array of dimension (n.comp,n.comp).
-        If None (default) then an array of normal r.v.'s is used.
+        Initial un-mixing array. If `w_init=None`, then an array of values
+        drawn from a normal distribution is used.
+
+    whiten_solver : {"eigh", "svd"}, default="svd"
+        The solver to use for whitening.
+
+        - "svd" is more stable numerically if the problem is degenerate, and
+          often faster when `n_samples <= n_features`.
+
+        - "eigh" is generally more memory efficient when
+          `n_samples >= n_features`, and can be faster when
+          `n_samples >= 50 * n_features`.
+
+        .. versionadded:: 1.2
 
     random_state : int, RandomState instance or None, default=None
         Used to initialize ``w_init`` when not specified, with a
@@ -289,6 +320,19 @@ def fastica(
     .. [1] A. Hyvarinen and E. Oja, "Fast Independent Component Analysis",
            Algorithms and Applications, Neural Networks, 13(4-5), 2000,
            pp. 411-430.
+
+    Examples
+    --------
+    >>> from sklearn.datasets import load_digits
+    >>> from sklearn.decomposition import fastica
+    >>> X, _ = load_digits(return_X_y=True)
+    >>> K, W, S = fastica(X, n_components=7, random_state=0, whiten='unit-variance')
+    >>> K.shape
+    (7, 64)
+    >>> W.shape
+    (7, 7)
+    >>> S.shape
+    (1797, 7)
     """
     est = FastICA(
         n_components=n_components,
@@ -299,11 +343,13 @@ def fastica(
         max_iter=max_iter,
         tol=tol,
         w_init=w_init,
+        whiten_solver=whiten_solver,
         random_state=random_state,
     )
-    S = est._fit(X, compute_sources=compute_sources)
+    est._validate_params()
+    S = est._fit_transform(X, compute_sources=compute_sources)
 
-    if est._whiten in ["unit-variance", "arbitrary-variance"]:
+    if est.whiten in ["unit-variance", "arbitrary-variance"]:
         K = est.whitening_
         X_mean = est.mean_
     else:
@@ -319,7 +365,7 @@ def fastica(
     return returned_values
 
 
-class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
+class FastICA(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
     """FastICA: a fast algorithm for Independent Component Analysis.
 
     The implementation is based on [1]_.
@@ -332,20 +378,20 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
         Number of components to use. If None is passed, all are used.
 
     algorithm : {'parallel', 'deflation'}, default='parallel'
-        Apply parallel or deflational algorithm for FastICA.
+        Specify which algorithm to use for FastICA.
 
-    whiten : str or bool, default="warn"
+    whiten : str or bool, default='unit-variance'
         Specify the whitening strategy to use.
-        If 'arbitrary-variance' (default), a whitening with variance arbitrary is used.
-        If 'unit-variance', the whitening matrix is rescaled to ensure that each
-        recovered source has unit variance.
-        If False, the data is already considered to be whitened, and no
-        whitening is performed.
 
-        .. deprecated:: 1.1
-            From version 1.3 whiten='unit-variance' will be used by default.
-            `whiten=True` is deprecated from 1.1 and will raise ValueError in 1.3.
-            Use `whiten=arbitrary-variance` instead.
+        - If 'arbitrary-variance', a whitening with variance
+          arbitrary is used.
+        - If 'unit-variance', the whitening matrix is rescaled to ensure that
+          each recovered source has unit variance.
+        - If False, the data is already considered to be whitened, and no
+          whitening is performed.
+
+        .. versionchanged:: 1.3
+            The default value of `whiten` changed to 'unit-variance' in 1.3.
 
     fun : {'logcosh', 'exp', 'cube'} or callable, default='logcosh'
         The functional form of the G function used in the
@@ -353,24 +399,39 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
         or 'cube'.
         You can also provide your own function. It should return a tuple
         containing the value of the function, and of its derivative, in the
-        point. Example::
+        point. The derivative should be averaged along its last dimension.
+        Example::
 
             def my_g(x):
                 return x ** 3, (3 * x ** 2).mean(axis=-1)
 
     fun_args : dict, default=None
         Arguments to send to the functional form.
-        If empty and if fun='logcosh', fun_args will take value
+        If empty or None and if fun='logcosh', fun_args will take value
         {'alpha' : 1.0}.
 
     max_iter : int, default=200
         Maximum number of iterations during fit.
 
     tol : float, default=1e-4
-        Tolerance on update at each iteration.
+        A positive scalar giving the tolerance at which the
+        un-mixing matrix is considered to have converged.
 
-    w_init : ndarray of shape (n_components, n_components), default=None
-        The mixing matrix to be used to initialize the algorithm.
+    w_init : array-like of shape (n_components, n_components), default=None
+        Initial un-mixing array. If `w_init=None`, then an array of values
+        drawn from a normal distribution is used.
+
+    whiten_solver : {"eigh", "svd"}, default="svd"
+        The solver to use for whitening.
+
+        - "svd" is more stable numerically if the problem is degenerate, and
+          often faster when `n_samples <= n_features`.
+
+        - "eigh" is generally more memory efficient when
+          `n_samples >= n_features`, and can be faster when
+          `n_samples >= 50 * n_features`.
+
+        .. versionadded:: 1.2
 
     random_state : int, RandomState instance or None, default=None
         Used to initialize ``w_init`` when not specified, with a
@@ -440,17 +501,34 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
     (1797, 7)
     """
 
+    _parameter_constraints: dict = {
+        "n_components": [Interval(Integral, 1, None, closed="left"), None],
+        "algorithm": [StrOptions({"parallel", "deflation"})],
+        "whiten": [
+            StrOptions({"arbitrary-variance", "unit-variance"}),
+            Options(bool, {False}),
+        ],
+        "fun": [StrOptions({"logcosh", "exp", "cube"}), callable],
+        "fun_args": [dict, None],
+        "max_iter": [Interval(Integral, 1, None, closed="left")],
+        "tol": [Interval(Real, 0.0, None, closed="left")],
+        "w_init": ["array-like", None],
+        "whiten_solver": [StrOptions({"eigh", "svd"})],
+        "random_state": ["random_state"],
+    }
+
     def __init__(
         self,
         n_components=None,
         *,
         algorithm="parallel",
-        whiten="warn",
+        whiten="unit-variance",
         fun="logcosh",
         fun_args=None,
         max_iter=200,
         tol=1e-4,
         w_init=None,
+        whiten_solver="svd",
         random_state=None,
     ):
         super().__init__()
@@ -462,9 +540,10 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
         self.max_iter = max_iter
         self.tol = tol
         self.w_init = w_init
+        self.whiten_solver = whiten_solver
         self.random_state = random_state
 
-    def _fit(self, X, compute_sources=False):
+    def _fit_transform(self, X, compute_sources=False):
         """Fit the model.
 
         Parameters
@@ -482,27 +561,8 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
         S : ndarray of shape (n_samples, n_components) or None
             Sources matrix. `None` if `compute_sources` is `False`.
         """
-        self._whiten = self.whiten
-
-        if self._whiten == "warn":
-            warnings.warn(
-                "From version 1.3 whiten='unit-variance' will be used by default.",
-                FutureWarning,
-            )
-            self._whiten = "arbitrary-variance"
-
-        if self._whiten is True:
-            warnings.warn(
-                "From version 1.3 whiten=True should be specified as "
-                "whiten='arbitrary-variance' (its current behaviour). This "
-                "behavior is deprecated in 1.1 and will raise ValueError in 1.3.",
-                FutureWarning,
-                stacklevel=2,
-            )
-            self._whiten = "arbitrary-variance"
-
         XT = self._validate_data(
-            X, copy=self._whiten, dtype=[np.float64, np.float32], ensure_min_samples=2
+            X, copy=self.whiten, dtype=[np.float64, np.float32], ensure_min_samples=2
         ).T
         fun_args = {} if self.fun_args is None else self.fun_args
         random_state = check_random_state(self.random_state)
@@ -522,18 +582,9 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
             def g(x, fun_args):
                 return self.fun(x, **fun_args)
 
-        else:
-            exc = ValueError if isinstance(self.fun, str) else TypeError
-            raise exc(
-                "Unknown function %r;"
-                " should be one of 'logcosh', 'exp', 'cube' or callable"
-                % self.fun
-            )
-
         n_features, n_samples = XT.shape
-
         n_components = self.n_components
-        if not self._whiten and n_components is not None:
+        if not self.whiten and n_components is not None:
             n_components = None
             warnings.warn("Ignoring n_components with whiten=False.")
 
@@ -545,15 +596,33 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
                 "n_components is too large: it will be set to %s" % n_components
             )
 
-        if self._whiten:
+        if self.whiten:
             # Centering the features of X
             X_mean = XT.mean(axis=-1)
             XT -= X_mean[:, np.newaxis]
 
             # Whitening and preprocessing by PCA
-            u, d, _ = linalg.svd(XT, full_matrices=False, check_finite=False)
+            if self.whiten_solver == "eigh":
+                # Faster when num_samples >> n_features
+                d, u = linalg.eigh(XT.dot(X))
+                sort_indices = np.argsort(d)[::-1]
+                eps = np.finfo(d.dtype).eps * 10
+                degenerate_idx = d < eps
+                if np.any(degenerate_idx):
+                    warnings.warn(
+                        "There are some small singular values, using "
+                        "whiten_solver = 'svd' might lead to more "
+                        "accurate results."
+                    )
+                d[degenerate_idx] = eps  # For numerical issues
+                np.sqrt(d, out=d)
+                d, u = d[sort_indices], u[:, sort_indices]
+            elif self.whiten_solver == "svd":
+                u, d = linalg.svd(XT, full_matrices=False, check_finite=False)[:2]
+
+            # Give consistent eigenvectors for both svd solvers
+            u *= np.sign(u[0])
 
-            del _
             K = (u / d).T[:n_components]  # see (6.33) p.140
             del u, d
             X1 = np.dot(K, XT)
@@ -579,13 +648,6 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
                     % {"shape": (n_components, n_components)}
                 )
 
-        if self.max_iter < 1:
-            raise ValueError(
-                "max_iter should be greater than 1, got (max_iter={})".format(
-                    self.max_iter
-                )
-            )
-
         kwargs = {
             "tol": self.tol,
             "g": g,
@@ -598,24 +660,20 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
             W, n_iter = _ica_par(X1, **kwargs)
         elif self.algorithm == "deflation":
             W, n_iter = _ica_def(X1, **kwargs)
-        else:
-            raise ValueError(
-                "Invalid algorithm: must be either `parallel` or `deflation`."
-            )
         del X1
 
         self.n_iter_ = n_iter
 
         if compute_sources:
-            if self._whiten:
+            if self.whiten:
                 S = np.linalg.multi_dot([W, K, XT]).T
             else:
                 S = np.dot(W, XT).T
         else:
             S = None
 
-        if self._whiten:
-            if self._whiten == "unit-variance":
+        if self.whiten:
+            if self.whiten == "unit-variance":
                 if not compute_sources:
                     S = np.linalg.multi_dot([W, K, XT]).T
                 S_std = np.std(S, axis=0, keepdims=True)
@@ -633,6 +691,7 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
 
         return S
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def fit_transform(self, X, y=None):
         """Fit the model and recover the sources from X.
 
@@ -651,8 +710,9 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
             Estimated sources obtained by transforming the data with the
             estimated unmixing matrix.
         """
-        return self._fit(X, compute_sources=True)
+        return self._fit_transform(X, compute_sources=True)
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def fit(self, X, y=None):
         """Fit the model to X.
 
@@ -670,7 +730,7 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
         self : object
             Returns the instance itself.
         """
-        self._fit(X, compute_sources=False)
+        self._fit_transform(X, compute_sources=False)
         return self
 
     def transform(self, X, copy=True):
@@ -694,9 +754,9 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
         check_is_fitted(self)
 
         X = self._validate_data(
-            X, copy=(copy and self._whiten), dtype=[np.float64, np.float32], reset=False
+            X, copy=(copy and self.whiten), dtype=[np.float64, np.float32], reset=False
         )
-        if self._whiten:
+        if self.whiten:
             X -= self.mean_
 
         return np.dot(X, self.components_.T)
@@ -719,9 +779,9 @@ class FastICA(_ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator)
         """
         check_is_fitted(self)
 
-        X = check_array(X, copy=(copy and self._whiten), dtype=[np.float64, np.float32])
+        X = check_array(X, copy=(copy and self.whiten), dtype=[np.float64, np.float32])
         X = np.dot(X, self.mixing_.T)
-        if self._whiten:
+        if self.whiten:
             X += self.mean_
 
         return X
