@@ -1,7 +1,4 @@
-"""
-The :mod:`sklearn.feature_extraction.image` submodule gathers utilities to
-extract features from images.
-"""
+"""Utilities to extract features from images."""
 
 # Authors: Emmanuelle Gouillart <emmanuelle.gouillart@normalesup.org>
 #          Gael Varoquaux <gael.varoquaux@normalesup.org>
@@ -240,6 +237,9 @@ def grid_to_graph(
     >>> mask[[1, 2], [1, 2], :] = True
     >>> graph = grid_to_graph(*shape_img, mask=mask)
     >>> print(graph)
+    <COOrdinate sparse matrix of dtype 'int64'
+      with 2 stored elements and shape (2, 2)>
+      Coords	Values
       (0, 0)    1
       (1, 1)    1
     """
