@@ -74,14 +74,20 @@ def _calculate_threshold(estimator, importances, threshold):
 def _estimator_has(attr):
     """Check if we can delegate a method to the underlying estimator.
 
-    First, we check the fitted estimator if available, otherwise we
-    check the unfitted estimator.
+    First, we check the fitted `estimator_` if available, otherwise we check the
+    unfitted `estimator`. We raise the original `AttributeError` if `attr` does
+    not exist. This function is used together with `available_if`.
     """
-    return lambda self: (
-        hasattr(self.estimator_, attr)
-        if hasattr(self, "estimator_")
-        else hasattr(self.estimator, attr)
-    )
+
+    def check(self):
+        if hasattr(self, "estimator_"):
+            getattr(self.estimator_, attr)
+        else:
+            getattr(self.estimator, attr)
+
+        return True
+
+    return check
 
 
 class SelectFromModel(MetaEstimatorMixin, SelectorMixin, BaseEstimator):
@@ -213,7 +219,7 @@ class SelectFromModel(MetaEstimatorMixin, SelectorMixin, BaseEstimator):
     >>> selector.estimator_.coef_
     array([[-0.3252...,  0.8345...,  0.4976...]])
     >>> selector.threshold_
-    0.55249...
+    np.float64(0.55249...)
     >>> selector.get_support()
     array([False,  True, False])
     >>> selector.transform(X)
@@ -343,12 +349,12 @@ class SelectFromModel(MetaEstimatorMixin, SelectorMixin, BaseEstimator):
         **fit_params : dict
             - If `enable_metadata_routing=False` (default):
 
-                Parameters directly passed to the `partial_fit` method of the
+                Parameters directly passed to the `fit` method of the
                 sub-estimator. They are ignored if `prefit=True`.
 
             - If `enable_metadata_routing=True`:
 
-                Parameters safely routed to the `partial_fit` method of the
+                Parameters safely routed to the `fit` method of the
                 sub-estimator. They are ignored if `prefit=True`.
 
                 .. versionchanged:: 1.4
@@ -507,8 +513,8 @@ class SelectFromModel(MetaEstimatorMixin, SelectorMixin, BaseEstimator):
         router = MetadataRouter(owner=self.__class__.__name__).add(
             estimator=self.estimator,
             method_mapping=MethodMapping()
-            .add(callee="partial_fit", caller="partial_fit")
-            .add(callee="fit", caller="fit"),
+            .add(caller="partial_fit", callee="partial_fit")
+            .add(caller="fit", callee="fit"),
         )
         return router
 
