@@ -1,11 +1,7 @@
-"""
-The :mod:`sklearn.kernel_approximation` module implements several
-approximate kernel feature maps based on Fourier transforms and Count Sketches.
-"""
+"""Approximate kernel feature maps based on Fourier transforms and count sketches."""
 
 # Author: Andreas Mueller <amueller@ais.uni-bonn.de>
 #         Daniel Lopez-Sanchez (TensorSketch) <lope@usal.es>
-
 # License: BSD 3 clause
 
 import warnings
